//----------------------------------------------
//
// "cube.tcl" example using C++ and "ToglMaster",
// Anargyros Sarafopoulos 13/10/05
// asarafop@bournemouth.ac.uk
//
//--------------------------------------------

/*
 * cube.tcl using ToglMaster 
 */

#include <iostream>
#include "cube.h"

using namespace std;

/*----------------------------------------------------------------------
 *
 * Static variables
 *
  ---------------------------------------------------------------------- */

// inti static variables of class Cube
float Cube::xAngle=0, Cube::yAngle=0, Cube::zAngle=0;
// display list id
static GLuint cubeList;

/*----------------------------------------------------------------------
 #
 # Cube::Clone
 #   Used to by Togl to create new instances of the "Cube" widget
 #
  ---------------------------------------------------------------------- */

ToglWidget* Cube::Clone() { return new Cube(); }


/*----------------------------------------------------------------------
 #
 # ToglWidget registers Tcl commands callback.
 #
  ---------------------------------------------------------------------- */

void Cube::RegisterTclCmds(Tcl_Interp *interp) {
		/* 
		 * Call Tcl_CreateCommand for application-specific commands, 
		 */
		Tcl_CreateCommand( 
				interp, 
				"getXrot", 
				Cube::getXrot_cb, 
				(ClientData)NULL,
				(Tcl_CmdDeleteProc *)NULL
		);
		Tcl_CreateCommand( 
				interp, 
				"getYrot",
				Cube::getYrot_cb,
				(ClientData)NULL,
				(Tcl_CmdDeleteProc *)NULL
		);
	}

/*----------------------------------------------------------------------
 #
 # Togl widget create callback.
 #
  ---------------------------------------------------------------------- */

void Cube::Init() {

	cubeList = glGenLists(1);
	glNewList(cubeList, GL_COMPILE);

	/* Front face */
	glBegin(GL_QUADS);
	glColor3f(0.0f, 0.7f, 0.1f);	/* Green */
	glVertex3f(-1.0f,  1.0f,  1.0f);
	glVertex3f( 1.0f,  1.0f,  1.0f);
	glVertex3f( 1.0f, -1.0f,  1.0f);
	glVertex3f(-1.0f, -1.0f,  1.0f);
	/* Back face */
	glColor3f(0.9f, 1.0f, 0.0f);   /* Yellow */
	glVertex3f(-1.0f,  1.0f, -1.0f);
	glVertex3f( 1.0f,  1.0f, -1.0f);
	glVertex3f( 1.0f, -1.0f, -1.0f);
	glVertex3f(-1.0f, -1.0f, -1.0f);
	/* Top side face */
	glColor3f(0.2f, 0.2f, 1.0f);   /* Blue */
	glVertex3f(-1.0f,  1.0f,  1.0f);
	glVertex3f( 1.0f,  1.0f,  1.0f);
	glVertex3f( 1.0f,  1.0f, -1.0f);
	glVertex3f(-1.0f,  1.0f, -1.0f);
	/* Bottom side face */
	glColor3f(0.7f, 0.0f, 0.1f);   /* Red */
	glVertex3f(-1.0f, -1.0f,  1.0f);
	glVertex3f( 1.0f, -1.0f,  1.0f);
	glVertex3f( 1.0f, -1.0f, -1.0f);
	glVertex3f(-1.0f, -1.0f, -1.0f);
	glEnd();

	glEndList();
}

/*----------------------------------------------------------------------
 #
 # Togl widget display callback.
 #
  ---------------------------------------------------------------------- */

void Cube::Display() {
	char *ident;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glLoadIdentity();	/* Reset modelview matrix to the identity matrix */
	glTranslatef(0.0f, 0.0f, -3.0f);      /* Move the camera back three units */
	glRotatef(xAngle, 1.0f, 0.0f, 0.0f);  /* Rotate by X, Y, and Z angles */
	glRotatef(yAngle, 0.0f, 1.0f, 0.0f);
	glRotatef(zAngle, 0.0f, 0.0f, 1.0f);
	 
	glEnable( GL_DEPTH_TEST );
	glCallList(cubeList);
	
	SwapBuffers();
}

/*----------------------------------------------------------------------
 #
 # Togl widget reshape callback.
 #
  ---------------------------------------------------------------------- */
void Cube::Reshape() {
	int width = GetWidth();
	int height = GetHeight();
	float aspect = (float) width / (float) height;

	/* Set Viewport */
	glViewport( 0, 0, width, height );

	/* Set up projection transform */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glFrustum(-aspect, aspect, -1.0, 1.0, 1.0, 10.0);

	/* Change back to model view transform for rendering */
	glMatrixMode(GL_MODELVIEW);
}

/*----------------------------------------------------------------------
 #
 # Cube:: tcl callbacks 
 #
  ---------------------------------------------------------------------- */
int Cube::getXrot_cb(
		ClientData clientData,
		Tcl_Interp *interp,
		int argc, const char *argv[]
) {
  sprintf( interp->result, "%d", (int)xAngle );
  return TCL_OK;
}

/*----------------------------------------------------------------------
 #
 # Cube:: tcl  callbacks 
 #
  ---------------------------------------------------------------------- */
int Cube::getYrot_cb( 
		ClientData clientData,
		Tcl_Interp *interp,
		int argc, const char *argv[]
) {
  sprintf( interp->result, "%d", (int)yAngle );
  return TCL_OK;
}


/*----------------------------------------------------------------------
 #
 # Init tcl/tk and togl
 #
  ---------------------------------------------------------------------- */

extern "C" EXPORT(int, Cube_Init)( Tcl_Interp *interp )
{
#ifdef USE_TCL_STUBS
  if (Tcl_InitStubs(interp, "8.1", 0) == NULL) {return TCL_ERROR;}
#endif
#ifdef USE_TK_STUBS
  if (Tk_InitStubs(interp, "8.1", 0) == NULL) {return TCL_ERROR;}
#endif

   if (Togl_Init(interp) == TCL_ERROR) {
      return TCL_ERROR;
   }

#ifdef macintosh
	 Togl_MacSetupMainInterp(interp);
#endif

	// init Togl library callbacks
	ToglMaster::InitTogl();
	// register ToglWidget classes
	ToglMaster::RegisterWidgetFunc("Cube", Cube::Clone);
	// register Tcl commands
	Cube::RegisterTclCmds(interp);

   return TCL_OK;
}

