//----------------------------------------------
//
// "cube.tcl" example using C++ and "ToglMaster",
// Anargyros Sarafopoulos 
// asarafop@bournemouth.ac.uk
//
//--------------------------------------------

#ifndef __CUBE_H__
#define __CUBE_H__

#include "toglmaster.h"
#include <cstdlib>
#include <cstring>


class Cube : public ToglWidget {
	static float xAngle, yAngle, zAngle;

	/* Create new Tcl Coommands */
	static int getXrot_cb(ClientData clientData, Tcl_Interp *interp, int argc, const char *argv[]);
	static int getYrot_cb(ClientData clientData, Tcl_Interp *interp, int argc, const char *argv[]);

	/*
	 * Extend Togl command
	 * command to set the X-axis rotation
	 */
	class setXrot : public ToglCmd {
		Cube *w;
	public:
		setXrot(std::string str, Cube *wgt) 
			: ToglCmd(str, *wgt), w(wgt)  { 
			;  // Ok
		}
		bool Cmd(int argc, const char *argv[]) {
			Tcl_Interp *interp = w->GetInterp();
			// error checking
			if (argc != 3) {
				throw WrongArgs(
					"wrong # args: should be \"pathName setXrot ?angle?\""
				);
				return false;
			}
			Cube::xAngle = atof( argv[2] );
			if ( Cube::xAngle < 0.0 ) {
				Cube::xAngle += 360.0;
			} else if ( Cube::xAngle > 360.0 ) {
			  Cube::xAngle -= 360.0;
			}

			w->PostRedisplay();
			// Let result string equal value
			strcpy( interp->result, argv[2] );
			return true;
		}
	} *CmdSetXrot;
	friend class setXrot;

	/*
	 * command to set the Y-axis rotation
	 */
	class setYrot : public ToglCmd {
		Cube *dw;
	public:
		setYrot(std::string str, Cube *w) 
			: ToglCmd(str, *w), dw(w) {
			; // Ok 
		}
		// make command "setYrot"
		bool Cmd(int argc, const char *argv[]) {
			Tcl_Interp *interp = dw->GetInterp();
			// error checking
			if (argc != 3) {
				throw WrongArgs(
					"wrong # args: should be \"pathName setYrot ?angle?\""
				);
				return false;
			}
			Cube::yAngle = atof( argv[2] );
			if ( Cube::yAngle < 0.0 ) {
			  Cube::yAngle += 360.0;
			} else if ( Cube::yAngle > 360.0 ) {
			  Cube::yAngle -= 360.0;
			}
			dw->PostRedisplay();
			// Let result string equal value
			strcpy( interp->result, argv[2] );
			return true;
		}
	}*CmdSetYrot;
	friend class setYrot;

public :
	Cube() {
		Cube::xAngle=0.0, Cube::yAngle=0.0, Cube::zAngle=0.0;
		CmdSetXrot = new setXrot("setXrot", this);
		CmdSetYrot = new setYrot("setYrot", this);
	}

	~Cube() { 
		delete CmdSetXrot;
		delete CmdSetYrot;
	}

	// Togl callbacks
	void Init();
	void Display();
	void Reshape();
	void Destroy() { ; } // do nothing
	void Timer()   { ; } // do nothing
	void Overlay() { ; } // do nothing
   
	// Clone and Tcl commands
	static ToglWidget* Clone();
	static void RegisterTclCmds(Tcl_Interp *interp); 
};

#endif // __CUBE_H__


