//----------------------------------------------
//
// "double.tcl" example using C++ and "ToglMaster",
// Anargyros Sarafopoulos 11/June/01
// asarafop@bournemouth.ac.uk
//
//--------------------------------------------

/*
 * double.tcl using ToglMaster 
 */

#include <iostream>
#include "double.h"

using namespace std;

/*----------------------------------------------------------------------
 *
 *  Init static members of Double 
 *
  ---------------------------------------------------------------------- */

GLuint Double::FontBase=0; // type of fonts to use
// where to print strings
GLfloat Double::CornerX=0, Double::CornerY=0, Double::CornerZ=0;  
// init rotation angles
float Double::xAngle=0.0, Double::yAngle=0.0, Double::zAngle=0.0;


/*----------------------------------------------------------------------
 #
 # Double::Clone
 #   Used to by Togl to create new instances of the "Double" widget
 #
  ---------------------------------------------------------------------- */

ToglWidget* Double::Clone() { return new Double(); }

/*----------------------------------------------------------------------
 #
 # Double::Cmd
 #    The "double" widgets accepts two commands, "setXrot" and "setYrot"
 #
  ---------------------------------------------------------------------- */
/*
bool Double::Cmd(int argc, const char *argv[]){
	if (strcmp(argv[1], "setXrot")==0) {
		setXrot_cb(argc, argv);
	} else if (strcmp(argv[1], "setYrot")==0) {
		setYrot_cb(argc, argv);
	}
	return (true);
}
*/

/*----------------------------------------------------------------------
 #
 # ToglWidget registers Tcl commands callback.
 #
  ---------------------------------------------------------------------- */

void Double::RegisterTclCmds(Tcl_Interp *interp) {
		/* 
		 * Call Tcl_CreateCommand for application-specific commands, 
		 * if they weren't already created by the init procedures 
		 * called above.
		 */
		Tcl_CreateCommand( 
				interp, 
				"getXrot", 
				Double::getXrot_cb, 
				(ClientData)NULL,
				(Tcl_CmdDeleteProc *)NULL
		);
		Tcl_CreateCommand( 
				interp, 
				"getYrot",
				Double::getYrot_cb,
				(ClientData)NULL,
				(Tcl_CmdDeleteProc *)NULL
		);
	}

/*----------------------------------------------------------------------
 #
 # Togl widget create callback.
 #
  ---------------------------------------------------------------------- */

void Double::Init() {

	FontBase = tkgl::LoadBitmapFont(TOGL_BITMAP_8_BY_13);
	if (!FontBase) {
		printf("Couldn't load font!\n");
		exit(1);
	}

/*
	// register Togl command arguments
	ToglMaster::RegisterCmdStrings("setXrot", "setYrot", NULL);
 */
}

/*----------------------------------------------------------------------
 #
 # Togl widget display callback.
 #
  ---------------------------------------------------------------------- */

void Double::Display() {
	static GLuint cubeList = 0;
	char *ident;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glLoadIdentity();	/* Reset modelview matrix to the identity matrix */
	glTranslatef(0.0f, 0.0f, -3.0f);      /* Move the camera back three units */
	glRotatef(xAngle, 1.0f, 0.0f, 0.0f);  /* Rotate by X, Y, and Z angles */
	glRotatef(yAngle, 0.0f, 1.0f, 0.0f);
	glRotatef(zAngle, 0.0f, 0.0f, 1.0f);
	 
	glEnable( GL_DEPTH_TEST );

	if (!cubeList) {
		cubeList = glGenLists(1);
		glNewList(cubeList, GL_COMPILE);

		/* Front face */
		glBegin(GL_QUADS);
		glColor3f(0.0f, 0.7f, 0.1f);	/* Green */
		glVertex3f(-1.0f,  1.0f,  1.0f);
		glVertex3f( 1.0f,  1.0f,  1.0f);
		glVertex3f( 1.0f, -1.0f,  1.0f);
		glVertex3f(-1.0f, -1.0f,  1.0f);
		/* Back face */
		glColor3f(0.9f, 1.0f, 0.0f);   /* Yellow */
		glVertex3f(-1.0f,  1.0f, -1.0f);
		glVertex3f( 1.0f,  1.0f, -1.0f);
		glVertex3f( 1.0f, -1.0f, -1.0f);
		glVertex3f(-1.0f, -1.0f, -1.0f);
		/* Top side face */
		glColor3f(0.2f, 0.2f, 1.0f);   /* Blue */
		glVertex3f(-1.0f,  1.0f,  1.0f);
		glVertex3f( 1.0f,  1.0f,  1.0f);
		glVertex3f( 1.0f,  1.0f, -1.0f);
		glVertex3f(-1.0f,  1.0f, -1.0f);
		/* Bottom side face */
		glColor3f(0.7f, 0.0f, 0.1f);   /* Red */
		glVertex3f(-1.0f, -1.0f,  1.0f);
		glVertex3f( 1.0f, -1.0f,  1.0f);
		glVertex3f( 1.0f, -1.0f, -1.0f);
		glVertex3f(-1.0f, -1.0f, -1.0f);
		glEnd();

		glEndList();

	}
	glCallList(cubeList);
	
	glDisable( GL_DEPTH_TEST );
	glLoadIdentity();
	glColor3f( 1.0, 1.0, 1.0 );
	glRasterPos3f( CornerX, CornerY, CornerZ );
	glListBase( FontBase );
	ident = GetIdent();
	if (strcmp(ident,"Double::single")==0) {
		print_string( "Single buffered" );
	}
	else {
		print_string( "Double buffered" );
	}
	SwapBuffers();
}

/*----------------------------------------------------------------------
 #
 # Togl widget reshape callback.
 #
  ---------------------------------------------------------------------- */
void Double::Reshape() {
	int width = GetWidth();
	int height = GetHeight();
	float aspect = (float) width / (float) height;

	glViewport( 0, 0, width, height );

	/* Set up projection transform */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glFrustum(-aspect, aspect, -1.0, 1.0, 1.0, 10.0);

	CornerX = -aspect;
	CornerY = -1.0f;
	CornerZ = -1.1f;

	/* Change back to model view transform for rendering */
	glMatrixMode(GL_MODELVIEW);
}


/*----------------------------------------------------------------------
 #
 # Double:: command  Togl callbacks 
 #
  ---------------------------------------------------------------------- */
/*
int Double::setXrot_cb(int argc, char *argv[]) {
   Tcl_Interp *interp = GetInterp();
   // error checking
   if (argc != 3) {
      Tcl_SetResult( interp,
                     "wrong # args: should be \"pathName setXrot ?angle?\"",
                     TCL_STATIC );
      return TCL_ERROR;
   }
   xAngle = atof( argv[2] );
	// printf( "before %f ", xAngle );
   if ( xAngle < 0.0 ) {
     xAngle += 360.0;
   } else if ( xAngle > 360.0 ) {
     xAngle -= 360.0;
   }

	// printf( "after %f \n", xAngle );
   PostRedisplay();
   // Let result string equal value
   strcpy( interp->result, argv[2] );
   return TCL_OK;
}
*/

/*----------------------------------------------------------------------
 #
 # Double:: command  Togl callbacks 
 #
  ---------------------------------------------------------------------- */
/*
int Double::setYrot_cb(int argc, char *argv[])
{
   Tcl_Interp *interp = GetInterp();

   // error checking
   if (argc != 3) {
      Tcl_SetResult( interp,
                     "wrong # args: should be \"pathName setYrot ?angle?\"",
                     TCL_STATIC );
      return TCL_ERROR;
   }

   yAngle = atof( argv[2] );
   
   if ( yAngle < 0.0 ) {
     yAngle += 360.0;
   } else if ( yAngle > 360.0 ) {
     yAngle -= 360.0;
   }

   PostRedisplay();

   // Let result string equal value
   strcpy( interp->result, argv[2] );
   return TCL_OK;
}
*/

/*----------------------------------------------------------------------
 #
 # Double:: tcl callbacks 
 #
  ---------------------------------------------------------------------- */
int Double::getXrot_cb(
		ClientData clientData,
		Tcl_Interp *interp,
		int argc, const char *argv[]
) {
  sprintf( interp->result, "%d", (int)xAngle );
  return TCL_OK;
}

/*----------------------------------------------------------------------
 #
 # Double:: tcl  callbacks 
 #
  ---------------------------------------------------------------------- */
int Double::getYrot_cb( 
		ClientData clientData,
		Tcl_Interp *interp,
		int argc, const char *argv[]
) {
  sprintf( interp->result, "%d", (int)yAngle );
  return TCL_OK;
}


/*----------------------------------------------------------------------
 #
 # Init tcl/tk and togl
 #
  ---------------------------------------------------------------------- */

extern "C" EXPORT(int,Double_Init)( Tcl_Interp *interp )
{
#ifdef USE_TCL_STUBS
  if (Tcl_InitStubs(interp, "8.1", 0) == NULL) {return TCL_ERROR;}
#endif
#ifdef USE_TK_STUBS
  if (Tk_InitStubs(interp, "8.1", 0) == NULL) {return TCL_ERROR;}
#endif

   if (Togl_Init(interp) == TCL_ERROR) {
      return TCL_ERROR;
   }

#ifdef macintosh
	 Togl_MacSetupMainInterp(interp);
#endif

	// init Togl library callbacks
	ToglMaster::InitTogl();
	// register ToglWidget classes
	ToglMaster::RegisterWidgetFunc("Double", Double::Clone);
	// register Tcl commands
	Double::RegisterTclCmds(interp);

   return TCL_OK;
}

