//----------------------------------------------
//
// "double.tcl" example using C++ and "ToglMaster",
// Anargyros Sarafopoulos 11/June/01
// asarafop@bournemouth.ac.uk
//
//--------------------------------------------

#ifndef __DOUBLE_H__
#define __DOUBLE_H__

#include "toglmaster.h"
#include <cstdlib>
#include <cstring>

//using namespace std;

static inline void print_string( const char *s ) {
   glCallLists( strlen(s), GL_UNSIGNED_BYTE, s );
}

class Double : public ToglWidget {
	static GLuint FontBase;
	static float xAngle, yAngle, zAngle;
	static GLfloat CornerX, CornerY, CornerZ;  // where to print strings 
	static int getXrot_cb(ClientData clientData, Tcl_Interp *interp, int argc, const char *argv[]);
	static int getYrot_cb(ClientData clientData, Tcl_Interp *interp, int argc, const char *argv[]);
/*
	int setXrot_cb(int argc, char *argv[]);
	int setYrot_cb(int argc, char *argv[]);
*/
	/*
	 * command to set the X-axis rotation
	 */
	class setXrot : public ToglCmd {
		Double *w;
	public:
		setXrot(std::string str, Double *wgt) 
			: ToglCmd(str, *wgt), w(wgt)  { 
			;  // Ok
		}
		bool Cmd(int argc, const char *argv[]) {
			Tcl_Interp *interp = w->GetInterp();
			// error checking
			if (argc != 3) {
				throw WrongArgs(
					"wrong # args: should be \"pathName setXrot ?angle?\""
				);
				return false;
			}
			Double::xAngle = atof( argv[2] );
			// printf( "before %f ", xAngle );
			if ( Double::xAngle < 0.0 ) {
				Double::xAngle += 360.0;
			} else if ( Double::xAngle > 360.0 ) {
			  Double::xAngle -= 360.0;
			}
			// printf( "after %f \n", xAngle );
			w->PostRedisplay();
			// Let result string equal value
			strcpy( interp->result, argv[2] );
			return true;
		}
	} *CmdSetXrot;
	friend class setXrot;

	/*
	 * command to set the Y-axis rotation
	 */
	class setYrot : public ToglCmd {
		Double *dw;
	public:
		setYrot(std::string str, Double *w) 
			: ToglCmd(str, *w), dw(w) {
			; // Ok 
		}
		// make command "setYrot"
		bool Cmd(int argc, const char *argv[]) {
			Tcl_Interp *interp = dw->GetInterp();
			// error checking
			if (argc != 3) {
				throw WrongArgs(
					"wrong # args: should be \"pathName setYrot ?angle?\""
				);
				return false;
			}
			Double::yAngle = atof( argv[2] );
			if ( Double::yAngle < 0.0 ) {
			  Double::yAngle += 360.0;
			} else if ( Double::yAngle > 360.0 ) {
			  Double::yAngle -= 360.0;
			}
			dw->PostRedisplay();
			// Let result string equal value
			strcpy( interp->result, argv[2] );
			return true;
		}
	}*CmdSetYrot;
	friend class setYrot;

public :
	Double() {
		CmdSetXrot = new setXrot("setXrot", this);
		CmdSetYrot = new setYrot("setYrot", this);
	}

	~Double() { 
		delete CmdSetXrot;
		delete CmdSetYrot;
	}

	// Togl callbacks

	void Init();
	void Display();
	void Reshape();
	void Destroy() { ; } // do nothing
	void Timer()   { ; } // do nothing
	void Overlay() { ; } // do nothing
	//bool Cmd(int argc, const char *argv[]);
   
	// Clone and Tcl commands

	static ToglWidget* Clone();
	static void RegisterTclCmds(Tcl_Interp *interp); 
};

#endif // __DOUBLE_H__

