//----------------------------------------------
//
// Anargyros Sarafopoulos 01/July/02
// asarafop@bournemouth.ac.uk
//
//--------------------------------------------

#ifndef __SCANLINE_H__
#define __SCANLINE_H__

#include "toglmaster.h"

// Macros
#define GRID_DX		20
#define GRID_DY		20

class Scanline : public ToglWidget {
	// polygon data
	int plg_01[9];
	int plg_02[9];
	int current_point_01;
	int current_point_02;

	/*
	 *  Move polygons' points 
	 */
	class movePoints: public ToglCmd {
		Scanline *w;
	public:
		movePoints(std::string str, Scanline *wgt) 
			: ToglCmd(str, *wgt), w(wgt)  {
			;  // Ok
		}
		bool Cmd(int argc, const char *argv[]) {
			Tcl_Interp *interp = w->GetInterp();
			// error checking
			if (argc != 4) {
				throw WrongArgs(
					"wrong # args: should be \"pathName movePoints x y\""
				);
				return false;
			}
			
			//printf( "Pixel co-ordinates == %d, %d\n", atoi(argv[2], argv[3]));
			int width  = w->GetWidth();
			int height = w->GetHeight();
			int x = (atoi(argv[2]))/GRID_DX;
			int y = (height - atoi(argv[3]))/GRID_DY;
			//printf( "grid co-ordinates == %d, %d\n", x, y);
			
			
			if (w->current_point_01 >=1 && w->current_point_01 <=4){
				w->plg_01[w->current_point_01*2-1]=x;
				w->plg_01[w->current_point_01*2-0]=y;
			}
			
			if (w->current_point_02 >=1 && w->current_point_02 <=4){
				w->plg_02[w->current_point_02*2-1]=x;
				w->plg_02[w->current_point_02*2-0]=y;
			}	
			w->PostRedisplay();
			// Let result string equal to the value: "x y"
			sprintf(interp->result, "%5d %5d", x, y);
			return true;
		}
	} *CmdMovePoints;
	friend class movePoints;
	
	
	/*
	 *  Select a point 
	 */
	class selectPoints: public ToglCmd {
		Scanline *w;
	public:
		selectPoints(std::string str, Scanline *wgt) 
			: ToglCmd(str, *wgt), w(wgt)  {
			;  // Ok
		}
		bool Cmd(int argc, const char *argv[]) {
			Tcl_Interp *interp = w->GetInterp();
			// error checking
			if (argc != 4) {
				throw WrongArgs(
					"wrong # args: should be \"pathName selectPoints x y\""
				);
				return false;
			}
			//printf( "Pixel co-ordinates == %d, %d\n", atoi(argv[2], argv[3]));
			int width  = w->GetWidth();
			int height = w->GetHeight();
			int x = (atoi(argv[2]))/GRID_DX;
			int y = (height - atoi(argv[3]))/GRID_DY;
			//printf( "grid co-ordinates == %d, %d\n", x, y);
			
			
			int p, i;
			w->current_point_01 = -1;
			for (p=1, i=0; p<=w->plg_01[0]; p++, i+=2) {
				if ((w->plg_01[i+1] == x) && (w->plg_01[i+2]==y)) {
					w->current_point_01 = p;
					//printf( "points == %d", p);
					break;
				}
			}
			
			w->current_point_02 = -1;
			for (p=1, i=0; p<=w->plg_02[0]; p++, i+=2) {
				if ((w->plg_02[i+1] == x) && (w->plg_02[i+2]==y)) {
					w->current_point_02 = p;
					break;
				}
			}
			
			// Let result string equal to the value "x y"
			sprintf(interp->result, "%d %d", x, y);
			return true;
		}
	} *CmdSelectPoints;
	friend class selectPoints;

public :
	Scanline() {
		plg_01[0]= 3;
		plg_01[1]= 2; plg_01[2]=18;
		plg_01[3]=18; plg_01[4]=13;
		plg_01[5]=10; plg_01[6]= 1;
		plg_01[7]= 0; plg_01[8]= 0;
		
		plg_02[0]= 3;
		plg_02[1]=20; plg_02[2]=22;
		plg_02[3]=18; plg_02[4]=13;
		plg_02[5]= 2; plg_02[6]=18;
		plg_02[7]= 0; plg_02[8]= 0;
		
		current_point_01 = current_point_02 = -1;
		CmdMovePoints    = new movePoints("movePoints", this);
		CmdSelectPoints  = new selectPoints("selectPoints", this);
	}

	~Scanline() { 
		delete CmdMovePoints;
		delete CmdSelectPoints;
	}

	// Togl callbacks
	void Init();
	void Display();
	void Reshape();
	void Timer();
	void Destroy() { ; } // do nothing
	void Overlay() { ; } // do nothing
   
	// Clone Widget and Tcl commands
	static ToglWidget* Clone();
};

#endif // __SCANLINE_H__

