
#include "GLfunctions.h"
//-------------------------------
//globals
//-------------------------------
vector<c_driver> devices;
c_visuals_01 visual_01;
c_visuals_02 visual_02;
int visualNo = 0;

//--------------------------------
//OPENGL FUNCTIONS
//--------------------------------

void draw()
{
	// clear the screen & depth buffer
	glClear(GL_DEPTH_BUFFER_BIT|GL_COLOR_BUFFER_BIT);

	// clear the previous transform
	glLoadIdentity();

	// set the camera position
	gluLookAt(	0,1,10,	//	eye pos
				0,0,0,	//	aim point
				0,1,0);	//	up direction

	
	//run visual
	if(visualNo == 0)
		visual_01.runVisual(&devices[0]);
	else if(visualNo == 1)
		visual_02.runVisual(&devices[0]);
	else
		cout << "No visual selected" << endl;


	// currently we've been drawing to the back buffer, we need
	// to swap the back buffer with the front one to make the image visible
	glutSwapBuffers();
}

void reshape(int w, int h)
{
	if (h==0) {
		h=1;
		}

	// set the drawable region of the window
	glViewport(0,0,w,h);

	// set up the projection matrix 
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	// just use a perspective projection
	gluPerspective(45,(float)w/h,0.1,100);

	// go back to modelview matrix so we can move the objects about
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

void idle()
{
	//This code was theoretically supposed to record more than one device, 
	//though I couldn't get it to work if i defined more than one device
	for(vector<c_driver>::iterator iter = devices.begin(); iter != devices.end();++iter)
	{
		iter->runDevice();

		iter->doVu(iter->getChannel());
		iter->printVu();

		glutPostRedisplay();
	}

	

	//keyboard functions

	if(IsKeyPressed(113))//key 'q'
		visualNo = 0;

	if(IsKeyPressed(119))//key 'w'
	{
		visualNo = 1;		
	}

}

//function called before main openGL loop
void onInit()
{
	//set keyboard functions
	InitKeyboard();

	//set values for position and material of light
	GLfloat mat_specular[] = { 1.0, 1.0, 1.0, 0.15 };
    GLfloat mat_shininess[] = { 100.0 };
    GLfloat position[] = { 0.5, 0.5, 1.0, 0.0 };

	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
	glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);	
	glLightfv(GL_LIGHT0, GL_POSITION, position);
	
	//Didnt use any textures in the end
	//glEnable(GL_TEXTURE_2D);

	//enable lights
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	//enable depth test
	glEnable(GL_DEPTH_TEST);
	//glEnable(GL_CULL_FACE);

	glShadeModel(GL_SMOOTH);
	
	//enable apha channels and blending
	//glEnable(GL_BLEND);
	//glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);	
	
	//drawing

	
	
	//fog
	glEnable(GL_FOG);
	
	//fog colour
	float fogColour[4] = {0.0, 0.0, 0.0, 1};
	glFogfv(GL_FOG_COLOR, fogColour);

	//fog blend factor
	glFogi(GL_FOG_MODE, GL_EXP);

	glFogf(GL_FOG_DENSITY, 0.05);

	glHint(GL_FOG_HINT, GL_DONT_CARE);
	


	//create c_driver object and set up input for this object
	c_driver tmp;
	devices.push_back(tmp);

		
	devices[0].loadDrivers();


	//theoretically, this code was supposed to perform the neccessary code for
	//more than one input device to be recorded, however, i have not made it
	//work yet


	char key = 0;
	int cnt = 0;
	//load first input device
	do
	{	
		devices[cnt].loadDevice();

		//add more devices?
		cout << "===============================================================" << endl;
		cout << "Press ""d"" to get another input device, any other key to continue" << endl;
		cout << "===============================================================" << endl;
		
		key = _getch();

		if(key == 100)
		{
			++cnt;
			c_driver tmp;
			devices.push_back(tmp);
		}
	}
	while(key == 100);
	
	//start recording devices
	for(vector<c_driver>::iterator iter = devices.begin(); iter != devices.end();++iter)
	{
		iter->setDevice();
		iter->startRecord();
	}
		

	//oscilloscope DSP unit
	//initDSP();
	
	



}

void onExit()
{
	for(vector<c_driver>::iterator iter = devices.begin(); iter != devices.end();++iter)
	{
		iter->closeDevice();
	}
}