#include "MLcode.h"

//PARAMETER OBJECT

c_parameter::c_parameter()
{
	MLint64 paramId = 0;
	MLint64 deviceId = 0;
	MLpv* paramName = NULL;
}

c_parameter::c_parameter(MLint64 i_paramId,MLint64 i_deviceId)
{
	paramId = i_paramId;
	deviceId = i_deviceId;

	MLpv* paramCap;

	if( mlPvGetCapabilities(deviceId,paramId,&paramCap) == ML_STATUS_INVALID_ID)
		cout << "Invalid Id" << endl;
	else
		paramName = mlPvFind( paramCap, ML_NAME_BYTE_ARRAY );
		

	mlFreeCapabilities(paramCap);

}


void c_parameter::setParameter(MLint64 i_paramId,MLint64 i_deviceId)
{
	paramId = i_paramId;
	deviceId = i_deviceId;

	MLpv* paramCap;

	if( mlPvGetCapabilities(deviceId,paramId,&paramCap) == ML_STATUS_INVALID_ID)
		cout << "Invalid Id" << endl;
	else
		paramName = mlPvFind( paramCap, ML_NAME_BYTE_ARRAY );
		 

	mlFreeCapabilities(paramCap);

}

void c_parameter::printParameter()
{
	cout << (char *) (paramName->value.pByte) << endl;
}



//PARAM IDS CLASS

c_params::c_params()
{
	numParams = 0;
	paramIds = NULL;
	pathId = 0;
	deviceId = 0;
}

c_params::c_params(MLint64 i_pathId)
{
	pathId = i_pathId;

	MLpv* pathCap;
	mlGetCapabilities( pathId, &pathCap );

	paramIds = mlPvFind( pathCap, ML_PARAM_IDS_INT64_ARRAY );
	pathName = mlPvFind( pathCap, ML_NAME_BYTE_ARRAY);
	numParams = paramIds->length;

	unsigned int i;
	for(i=0;i<numParams;i++)
	{
		c_parameter t_param(paramIds->value.pInt64[i],pathId);		
		parameter.push_back(t_param);

		parameter[i].printParameter();
	}	

	mlFreeCapabilities( pathCap );
}


void c_params::loadParams(MLint64 i_pathId)
{
	pathId = i_pathId;

	MLpv* pathCap;
	mlGetCapabilities( pathId, &pathCap );

	paramIds = mlPvFind( pathCap, ML_PARAM_IDS_INT64_ARRAY );
	pathName = mlPvFind( pathCap, ML_NAME_BYTE_ARRAY);
	numParams = paramIds->length;

	
	unsigned int i;
	for(i=0;i<numParams;i++)
	{
		c_parameter t_param(paramIds->value.pInt64[i],pathId);		
		parameter.push_back(t_param);

		parameter[i].printParameter();
	}	

	mlFreeCapabilities( pathCap );
}


void c_params::printNumParams()
{
	cout << "Path supports " << numParams << " parameters" << endl;	

}

void c_params::printPathName()
{
	cout << "Path: " << (char *) (pathName->value.pByte) << endl;
}

unsigned int c_params::getNumParams()
{
	return numParams;
}

MLint64 c_params::getParamId(unsigned int paramNo)
{
	if(paramNo >= 0 && paramNo<numParams)
		return paramIds->value.pInt64[paramNo];
	else{
		cout << "Invalid parameter number" << endl;
		return 0;}
}


//PATH IDS CLASS
c_paths::c_paths()
{
	numPaths = 0;
	pathIds  = NULL;
	deviceId = 0;
	deviceName = NULL;
}

c_paths::c_paths(MLint64 i_deviceId)
{
	deviceId = i_deviceId;
	
	MLpv* deviceCap;
	mlGetCapabilities( deviceId, &deviceCap );

	pathIds = mlPvFind( deviceCap, ML_DEVICE_PATH_IDS_INT64_ARRAY );
	numPaths = pathIds->length;
	deviceName = mlPvFind( deviceCap, ML_NAME_BYTE_ARRAY );	

	
	unsigned int i;
	for(i=0;i<numPaths;i++)
	{
		c_params t_params(pathIds->value.pInt64[i]);
		paramIds.push_back(t_params);
		
		paramIds[i].printPathName();
		paramIds[i].printNumParams();
	}
	
	mlFreeCapabilities( deviceCap );
}

void c_paths::loadPaths(MLint64 i_deviceId)
{
	deviceId = i_deviceId;
	
	MLpv* deviceCap;
	mlGetCapabilities( deviceId, &deviceCap );

	pathIds = mlPvFind( deviceCap, ML_DEVICE_PATH_IDS_INT64_ARRAY );
	numPaths = pathIds->length;	
	deviceName = mlPvFind( deviceCap, ML_NAME_BYTE_ARRAY );

	
	unsigned int i;
	for(i=0;i<numPaths;i++)
	{
		c_params t_params(pathIds->value.pInt64[i]);
		paramIds.push_back(t_params);
		
		paramIds[i].printPathName();
		paramIds[i].printNumParams();
	}
	
	mlFreeCapabilities( deviceCap );
}


void c_paths::printNumPaths()
{
	
	cout << "Device has " << numPaths << "paths." << endl;

}

void c_paths::printDevice()
{
	cout << "Device: " << (char *) (deviceName->value.pByte) << endl;
}

MLint64 c_paths::getPathId(unsigned int pathNo)
{
	if(numPaths > pathNo && pathNo >= 0)
		return pathIds->value.pInt64[pathNo];
	else
	{
		cout << "Invalid device number" << endl;
		return 0;
	}
}

MLint32 c_paths::getPathBufferAlignment(unsigned int pathNo)
{
	
	
	if(numPaths > pathNo && pathNo >= 0)
	{
		MLpv* pathCap;
		mlGetCapabilities(pathIds->value.pInt64[pathNo],&pathCap);
		MLpv* buffAlignment = mlPvFind(pathCap,ML_PATH_BUFFER_ALIGNMENT_INT32);		
		
		return buffAlignment->value.int32;
	}
	else
	{
		cout << "Invalid device number" << endl;
		return 0;
	}
}

unsigned int c_paths::getNumPaths()
{
	return numPaths;
}



//JACK IDS CLASS

c_jackName::c_jackName()
{
	jackName = NULL;

}

c_jackName::c_jackName(MLint64 i_jackId)
{
	jackId = i_jackId;

	MLpv* jackCap;
	mlGetCapabilities(jackId, &jackCap);

	jackName = mlPvFind(jackCap,ML_NAME_BYTE_ARRAY);

}

void c_jackName::printJackName()
{
	cout << "jack: " << (char *) (jackName->value.pByte) << endl;
}

c_jacks::c_jacks()
{
	numJacks = 0;
	jackIds = NULL;
	deviceId = 0;
	
}

c_jacks::c_jacks(MLint64 i_deviceId)
{
	deviceId = i_deviceId;

	MLpv* deviceCap;
	mlGetCapabilities( deviceId, &deviceCap );

	jackIds = mlPvFind( deviceCap, ML_DEVICE_JACK_IDS_INT64_ARRAY );
	numJacks = jackIds->length;	
	

	for(unsigned int i = 0; i < numJacks; i++ )
	{
		c_jackName t_jackName(jackIds->value.pInt64[i]);
		names.push_back(t_jackName);

		names[i].printJackName();
	}

	mlFreeCapabilities(deviceCap);
}







//DEVICE IDS CLASS


c_devices::c_devices()
{
	MLpv* systemCap;
	//get system capabilities
	mlGetCapabilities( ML_SYSTEM_LOCALHOST, &systemCap );

	deviceIds = mlPvFind( systemCap, ML_SYSTEM_DEVICE_IDS_INT64_ARRAY );
	numDevices = deviceIds->length;

	
	unsigned int i;
	for(i=0;i<numDevices;i++)
	{
		c_paths t_paths(deviceIds->value.pInt64[i]);
		paths.push_back(t_paths);
		
		paths[i].printDevice();
		paths[i].printNumPaths();


		c_jacks t_jacks(deviceIds->value.pInt64[i]);
		jacks.push_back(t_jacks);
		
	}

		
	mlFreeCapabilities( systemCap );
}

void c_devices::loadDevices()
{
	MLpv* systemCap;
	//get system capabilities
	mlGetCapabilities( ML_SYSTEM_LOCALHOST, &systemCap );

	deviceIds = mlPvFind( systemCap, ML_SYSTEM_DEVICE_IDS_INT64_ARRAY );
	numDevices = deviceIds->length;

	unsigned int i;
	for(i=0;i<numDevices;i++)
	{
		c_paths t_paths(deviceIds->value.pInt64[i]);
		paths.push_back(t_paths);
		
		//paths[i].printDevice();
	}
		
	mlFreeCapabilities( systemCap );
}

void c_devices::printNumDevices()
{
	
	cout << "There are " << numDevices << " devices" << endl;	

}

void c_devices::printAll()
{
	cout << "There are " << numDevices << " devices" << endl;

	//unsigned int i;
	for(vector<c_paths>::iterator iter = paths.begin();iter != paths.end();iter++)
	{				
	
		iter->printDevice();
		iter->printNumPaths();
		
		
		//cout << endl;
		/*unsigned int j;
		for(j=0;j<paths[i].getNumPaths();j++)
		{
					
			paths[i].paramIds[j].printPathName();
			paths[i].paramIds[j].printNumParams();
			cout << endl;

			unsigned int k;
			for(k=0;k<paths[i].paramIds[j].getNumParams();k++)
			{
				paths[i].paramIds[j].parameter[k].printParameter();
			}
			cout << endl;

		}*/
	}
}

MLint64 c_devices::getDeviceId(unsigned int deviceNo)
{
	if(numDevices > deviceNo && deviceNo >= 0)
		return deviceIds->value.pInt64[deviceNo];
	else
	{
		cout << "Invalid device number" << endl;
		return 0;
	}
}

unsigned int c_devices::getNumDevices()
{
	return numDevices;
}








void openMLcode()
{
c_devices devices;

	cout << endl << endl;
	
	MLint64 devId  = devices.getDeviceId(1);

	MLint64 jackId = 0;
	MLint64 pathId = devices.paths[1].getPathId(0); //devices.paths[1].getPathId(1);

	MLopenid openPath = 0;

	
		
	
/*
	MLint32* memAlignment; //= devices.paths[1].getPathBufferAlignment(1);

	MLpv* pathCap;
	mlGetCapabilities(devices.paths[1].getPathId(0),&pathCap);
	devices.paths[1].paramIds[0].printPathName();
	MLpv* comAlignment = mlPvFind(pathCap,ML_PATH_COMPONENT_ALIGNMENT_INT32);		
		
	memAlignment = comAlignment->value.pInt32;
*/


	//find device, connect to jack,jack to path
	//cout << "Enter device by name:" << endl;
	
	//string device;	
	//cin >> device;

	//char* chDevice = &device
	
	
	//mluFindDeviceByName( ML_SYSTEM_LOCALHOST, device.c_str(), &devId );
	
	

	if(mluFindFirstInputJack( devId, &jackId ) == ML_STATUS_NO_ERROR)
		cout << "ok" << endl;
	else
		cout << "error" << endl;

	
	
	/*
	if(mluFindPathToJack( jackId, &pathId, memAlignment ) == ML_STATUS_NO_ERROR)
		cout << "ok" << endl;
	else
		if(mluFindPathToJack( jackId, &pathId, memAlignment ) == ML_STATUS_INVALID_ARGUMENT)
			cout << "invalid argument" << endl;
	else
		cout <<"error"<<endl;
*/

	
	
	//open path
	if(mlOpen( pathId, NULL, &openPath ) == ML_STATUS_NO_ERROR)
		cout << "ok" << endl;
	else
		cout << "error" << endl;

	
	//set up path
	//array of parameter value pairs
	MLpv controls[4];
	MLreal64 gain = -12; //decibels
	
	
	controls[0].param = ML_AUDIO_FORMAT_INT32;
	controls[0].value.int32 = ML_AUDIO_FORMAT_S16;

	controls[1].param = ML_AUDIO_CHANNELS_INT32;
	controls[1].value.int32 = 1;

	controls[2].param = ML_AUDIO_SAMPLE_RATE_REAL64;
	controls[2].value.real64 =  44100.0;
	
	controls[3].param = ML_END;//always ends with ML_END

	//set controls on audio path
	
	//open path
	if(mlSetControls( openPath, controls ) == ML_STATUS_NO_ERROR)
		cout << "ok" << endl;
	else
		cout << "error" << endl;

	//send buffers to device for processing

	MLpv message[2];
	MLbyte* audioBuffer;
	audioBuffer = new MLbyte(32);

	message[0].param = ML_AUDIO_BUFFER_POINTER;
	message[0].value.pByte = audioBuffer;
	message[0].length = sizeof(audioBuffer);
	message[1].param = ML_END;

	//queue up buffers
	if(mlSendBuffers(openPath,message) == ML_STATUS_NO_ERROR)
		cout << "ok" << endl;
	else
		cout << "error" << endl;


	//make wait handle to indicate buffer transfer has finished
	MLwaitable pathWaitHandle;
	mlGetReceiveWaitHandle( openPath, &pathWaitHandle );
	
	//process enqued messages
	
	if(mlBeginTransfer(openPath) == ML_STATUS_NO_ERROR)
		cout << "ok" << endl;
	else
		cout << "error" << endl;
	

	//wait for call to 'fire'
	WaitForSingleObject( pathWaitHandle, INFINITE );


	//once call has fired, a reply will be waiting
	//receive reply message
	MLint32 messageType;
	MLpv* replyMessage;

	MLint32 count;
	mlGetSendMessageCount(openPath,&count);
	cout << "Sent messages: " << count << endl;

	mlGetReceiveMessageCount(openPath,&count);
	cout << "Received messages: " << count << endl;	


	mlReceiveMessage( openPath, &messageType, &replyMessage);

	//confirm successful buffer transfer
	if( messageType == ML_BUFFERS_COMPLETE )
		cout << "buffers transferred" << endl;
	else
		cout << "buffers not transferred" << endl;

	//reply message has same structure as original buffer/message sent to device
	//so following can be retrieved
	MLbyte* audiobuffer =	replyMessage[0].value.pByte;
	cout << audiobuffer << endl;
	//MLint64 audioUST	=	replyMessage[1].value.int64;

	
	//refill the buffer for further processing
	mlSendBuffers( openPath, replyMessage );


	//end transfer
	mlEndTransfer(openPath);

	
	mlClose(openPath);
	

	//devices.printAll();	

	//devices.printNumDevices();

	/*unsigned int i;
	for(i=0;i<devices.getNumDevices();i++)
	{
		devices.paths[i].printDevice();
	}*/
	
	/*
	devices.printNumDevices();

	unsigned int i;
	for(i=0;i < devices.getNumDevices();i++)
	{				
		
		c_paths path(devices.getDeviceId(i));
		path.printDevice();
		path.printNumPaths();
		cout << endl;

		unsigned int j;
		for(j=0;j<path.getNumPaths();j++)
		{
					
			c_params paramIds(path.getPathId(j));
			paramIds.printPathName();
			paramIds.printNumParams();
			cout << endl;


			unsigned int k;
			for(k=0;k<paramIds.getNumParams();k++)
			{
				c_parameter param(paramIds.getParamId(k),path.getPathId(j));
				param.printParameter();
			}
			cout << endl;

		}
	}*/

	


/*	
	c_params path0_params(dev1_paths.getPathId(0));

	path0_params.printNumParams();

	unsigned int i;
	for(i=0;i<path0_params.retNumParams();i++)
	{
		//cout << path0_params.getParamId(i) << endl;

		c_parameter parameter(path0_params.getParamId(i),dev1_paths.getPathId(0));

		parameter.printParameter();


	}


	
	//c_parameter path0_parameterName0(path0_params.getParamId(0),devices.getDeviceId(1));
	//path0_parameterName0.printParameter();


	//unsigned int i;
	//for(i=0;i<path0_params.retNumParams();i++)
	//{

	//path0_parameterName0.setParameter(path0_params.getParamId(0),devices.getDeviceId(1));
	
	
	//}
*/

	
	
	
	
	
/*	
	MLpv* systemCap;
	mlGetCapabilities( ML_SYSTEM_LOCALHOST, &systemCap );

	MLpv* deviceIds = mlPvFind( systemCap, ML_SYSTEM_DEVICE_IDS_INT64_ARRAY );
		
	mlFreeCapabilities( systemCap );

	
	unsigned int numDevices = deviceIds->length;
	cout << "There are " << numDevices << " devices" << endl << endl << endl;




	unsigned int j,k, numPaths, numTranscoders;
	MLpv* deviceCap, *pathIds, *xcodeIds;

	

	if(numDevices>0)
	{
		for( j=0; j<numDevices; j++)
		{
			cout << "Device" << j << " has ID: " << deviceIds->value.pInt64[j] << endl;
			
			
			mlGetCapabilities( deviceIds->value.pInt64[j], &deviceCap );

			pathIds = mlPvFind( deviceCap, ML_DEVICE_PATH_IDS_INT64_ARRAY );
			xcodeIds = mlPvFind( deviceCap, ML_DEVICE_XCODE_IDS_INT64_ARRAY );
			MLpv* deviceNames = mlPvFind( deviceCap, ML_NAME_BYTE_ARRAY );

			mlFreeCapabilities( deviceCap );

			//print names
			cout << (char *) (deviceNames->value.pByte) << endl;

			numPaths = pathIds->length;
			numTranscoders = xcodeIds->length;

			cout << "Device supports " << numPaths << " paths and " << numTranscoders << " Transcoders"<< endl << endl;
		
			if(numPaths>0)
			{
				for(k = 0; k < numPaths; k++ )
				{
					cout << "I/O path " << k << " has ID: " << pathIds->value.pInt64[k] << endl;	


					MLpv* pathCap, *paramIds;
					mlGetCapabilities( pathIds->value.pInt64[k], &pathCap );

					paramIds = mlPvFind( pathCap, ML_PARAM_IDS_INT64_ARRAY );
					MLpv* pathNames = mlPvFind( pathCap, ML_NAME_BYTE_ARRAY);

					mlFreeCapabilities( pathCap );

					cout << (char *) (pathNames->value.pByte) << endl;


					unsigned int numParams = paramIds->length;
					

					cout << "Path supports " << numParams << " parameters" << endl;

					if(numParams > 0)
					{
						unsigned int l;
						for(l=0;l<numParams;l++)
						{
							cout << "Parameter " << l << " ID: " << paramIds->value.pInt64[l] << endl;
						
								
							//char* buffer = NULL;
							//if( mlPvToString( deviceIds->value.pInt64[j], paramIds, buffer, &paramIds.length) )
							//	cout << "mlPvToString" << endl;

							//if( buffer != NULL )
							//	cout << "name: " << buffer << endl;

							
							MLpv* paramCap;						

							cout << deviceIds->value.pInt64[j] << endl;
							
							//deviceIds not workin
							if( mlPvGetCapabilities( deviceIds->value.pInt64[j], paramIds->value.pInt64[l],  &paramCap ) 
								== ML_STATUS_INVALID_ID )
								cout << "invalid id" << endl;
							else
								cout << "ok" << endl;
							
							//	cout << (char *) (paramNames->value.pByte) << endl;

							
							//MLpv* paramNames = mlPvFind( paramCap, ML_NAME_BYTE_ARRAY );

							//

												 
							//if(paramName != NULL)
							//	cout << "Parameter has name: " << (char*) (paramName->value.pByte) << endl;

							mlFreeCapabilities( paramCap );
						}
					}
					cout << endl;



				}			

			}

			cout << endl << endl;
			
		}
	}



	return 0;



/*
	MLint64 devId  = 0;
	MLint64 jackId = 0;
	MLint64 pathId = 0;

	MLopenid openPath = 0;

	//MLint32 retPathAlignment = 0;

	//find device, connect to jack,jack to path
	mluFindDeviceByName( ML_SYSTEM_LOCALHOST, argv[1], &devId );
	mluFindFirstOutputJack( devId, &jackId );
	mluFindPathToJack( jackId, &pathId, memoryAlignment );

	//open path
	mlOpen( devId, NULL, &openPath );

	//set up path
	//array of parameter value pairs
	MLpv controls[5];
	MLreal64 gain = -12; //decibels
	
	
	controls[0].param = ML_AUDIO_FORMAT_INT32;
	controls[0].value.int32 = ML_AUDIO_FORMAT_S16;

	controls[1].param = ML_AUDIO_CHANNELS_INT32;
	controls[1].value.int32 = 1;

	controls[2].param = ML_AUDIO_GAINS_REAL64_ARRAY;
	controls[2].value.pReal64 = &gain ;
	controls[2].length = 1 ;

	controls[3].param = ML_AUDIO_SAMPLE_RATE_REAL64;
	controls[3].value.real64 =  44100.0;
	
	controls[4].param = ML_END;//always ends with ML_END

	//set controls on audio path
	mlSetControls( openPath, controls );



	//send buffers to device for proceessing
	int i;
	for(i = 0; i<12; i++)
	{
		MLpv msg[3];

		msg[0].param = ML_AUDIO_BUFFER_POINTER;
		msg[0].value.pByte = (MLbyte*)buffers[i];
		msg[0].maxLength = imageSize;

		msg[1].param = ML_AUDIO_UST_INT64;

		msg[2].param = ML_END;

		//send message to open path
		mlSendBuffers( openPath, msg );
	}

	
	//begin message processing
	mlBeginTransfer(openPath);



	//make wait handle to indicate buffer transfer has finished
	MLwaitable pathWaitHandle;
	mlGetReceiveWaitHandle( openPath, &pathWaitHandle );

	//wait for call to 'fire'
	WaitForSingleObject( pathWaitHandle, INFINITE );


	//once call has fired, a reply will be waiting
	//receive reply message
	MLint32 messageType;
	MLpv* replyMessage;

	mlReceiveMessage( openPath, &messageType, &replyMessage);

	//confirm successful buffer transfer
	if( messageType == ML_BUFFERS_COMPLETE )
		cout << "buffers transferred" << endl;

	//reply message has same structure as original buffer/message sent to device
	//so following can be retrieved
	MLbyte* audiobuffer =	replyMessage[0].value.pByte;
	MLint64 audioUST	=	replyMessage[1].value.int64;

	
	//refill the buffer for further processing
	mlSendBuffers( openPath, replyMessage );


	//end the transfer
	mlEndTransfer(openPath);


	//close path
	mlClose(openPath);
*/
}